import QtQuick 1.1

// application-modal dialog
// Write sub-class with visual elements of dialog
// call show() to start modal
// call hide() to end
// call startTimeout(seconds) to start a timer which emits timedOut() signal
// Set dismissAnywhere and dismissable properties to allow dialog to be dismissed when touching screen anywhere.
// Derived elements may use dismissable property to emit dismissed() signal only in their own MouseArea handler.
// Set isTransparent property to allow passing mouse events to underlying controls

Item {
    id: root
    property bool isTop: true
    property bool isTransparent: false
    property bool dismissable: false
    property bool dismissAnywhere: false
    property alias isCounting:countDownTimer.running
    property bool blockKeys: false

    signal timedOut()
    signal triggered()
    signal dismissed()

    function startTimeout(seconds) {
        countDown = seconds
        countDownTimer.start()
    }

    function stopTimer() {
        countDownTimer.stop()
    }

    function getRootItem(child) {

        var top
        top = child

        if( !child )
            return child

        while( top.parent ) {
            top = top.parent
        }
        return top
    }

    function show() {
        visible = true
        mouse_area.parent = getRootItem(root)
        mouse_area.anchors.fill = mouse_area.parent
    }

    function hide() {
        mouse_area.parent = root
        visible = false
        countDownTimer.stop()
    }

    onDismissed: hide()

    Component.onCompleted: {
        if(isTop)
            root.parent = getRootItem(root)
    }

    property int countDown

    z: 99

    // blocking mouse area over entire root item
    MouseArea {
        id: mouse_area
        z: root.z - 1 // don't cover our children
        anchors.fill: parent

        onPressed: {
            if(root.dismissable && root.dismissAnywhere) {
                root.dismissed()
            }

            if(isTransparent){
                mouse.accepted = false
            }
        }
    }

    Timer {
        id: countDownTimer

        running: false
        interval: 1000
        repeat: true
        onTriggered: {
            countDown--
            if ( countDown <= 0 ) {
                running = false
                root.timedOut()
            }
            else
            {
                root.triggered()
            }
        }
    }
}
